//  ************************************************************************************************
//
//  BornAgain: simulate and fit reflection and scattering
//
//! @file      GUI/Views/SampleDesigner/LayerEditorUtils.h
//! @brief     Defines class LayerEditorUtils
//!
//! @homepage  http://www.bornagainproject.org
//! @license   GNU General Public License v3 or higher (see COPYING)
//! @copyright Forschungszentrum Jülich GmbH 2021
//! @authors   Scientific Computing Group at MLZ (see CITATION, AUTHORS)
//
//  ************************************************************************************************

#ifndef BORNAGAIN_GUI_VIEWS_SAMPLEDESIGNER_LAYEREDITORUTILS_H
#define BORNAGAIN_GUI_VIEWS_SAMPLEDESIGNER_LAYEREDITORUTILS_H

#include "GUI/Models/DoubleDescriptor.h"

#include <QColor>
#include <QList>
#include <functional>

class QLabel;
class DoubleSpinBox;
class QGridLayout;
class SampleEditorController;
class VectorDescriptor;
class QString;
class LayerZeroRoughnessItem;
class LayerBasicRoughnessItem;
class FTDecayFunction1DItem;
class FormFactorItem;
class FTDistribution1DItem;
class RotationItem;
class QWidget;
class ItemWithParticles;
class QPushButton;

//! Utility functions to support layer oriented sample editor
namespace LayerEditorUtils {

void updateLabelUnit(QLabel* label);
void updateLabelUnit(QLabel* label, DoubleSpinBox* editor);

void addMultiPropertyToGrid(QGridLayout* m_gridLayout, int firstCol,
                            const DoubleDescriptors& valueDescriptors, SampleEditorController* ec,
                            bool vertically, bool addSpacer);

void addMultiPropertyToGrid(QGridLayout* m_gridLayout, int firstCol,
                            const DoubleDescriptors& valueDescriptors, SampleEditorController* ec,
                            bool addSpacer);

void addVectorToGrid(QGridLayout* m_gridLayout, int firstCol, const VectorDescriptor& v,
                     SampleEditorController* ec, bool vertically, bool addSpacer);

QLabel* createBoldLabel(const QString& text);

DoubleDescriptors doubleDescriptorsOfItem(
    const std::variant<LayerZeroRoughnessItem*, LayerBasicRoughnessItem*>& item);

DoubleDescriptors doubleDescriptorsOfItem(FTDecayFunction1DItem* item);
DoubleDescriptors doubleDescriptorsOfItem(FormFactorItem* item);
DoubleDescriptors doubleDescriptorsOfItem(FTDistribution1DItem* item);
DoubleDescriptors doubleDescriptorsOfItem(RotationItem* item);

QWidget* createWidgetForItemWithParticles(QWidget* parentWidget,
                                          ItemWithParticles* itemWithParticles,
                                          SampleEditorController* ec, bool allowRemove = true);

QPushButton* createAddParticleButton(QWidget* parentWidget,
                                     std::function<void(const QString&)> slot);

QList<QColor> predefinedLayerColors();

} // namespace LayerEditorUtils

#endif // BORNAGAIN_GUI_VIEWS_SAMPLEDESIGNER_LAYEREDITORUTILS_H
